@@ -7,69 +7,33 @@
 #include <sys/types.h>
 #include <stdlib.h>
 
+#ifdef HAVE_BLKID_BLKID_H
+#include <blkid/blkid.h>
+#else
 #include <blkid.h>
+#endif
 
 #include "blkdev.h"
 #include "canonicalize.h"
 #include "pathnames.h"
 #include "fsprobe.h"
 
-#if defined(HAVE_BLKID_EVALUATE_TAG) || defined(HAVE_LIBVOLUME_ID)
-/* ask kernel developers why we need such ugly open() method... */
-static int
-open_device(const char *devname)
-{
-	int retries = 0;
-
-	do {
-		int fd = open(devname, O_RDONLY);
-		if (fd >= 0)
-			return fd;
-		if (errno != ENOMEDIUM)
-			break;
-		if (retries >= CRDOM_NOMEDIUM_RETRIES)
-			break;
-		++retries;
-		sleep(3);
-	} while(1);
-
-	return -1;
-}
-#endif
+static blkid_cache blcache;
 
 /*
  * Parses NAME=value, returns -1 on parse error, 0 success. The success is also
  * when the 'spec' doesn't contain name=value pair (because the spec could be
  * a devname too). In particular case the pointer 'name' is set to NULL.
-
- * The result is a new allocated string (the 'name' pointer).
  */
 int
 fsprobe_parse_spec(const char *spec, char **name, char **value)
 {
-	char *vl, *tk, *cp;
-
 	*name = NULL;
 	*value = NULL;
 
-	if (!(cp = strchr(spec, '=')))
-		return 0;				/* no name= */
+	if (strchr(spec, '='))
+		return blkid_parse_tag_string(spec, name, value);
 
-	tk = strdup(spec);
-	vl = tk + (cp - spec);
-	*vl++ = '\0';
-
-	if (*vl == '"' || *vl == '\'') {
-		if (!(cp = strrchr(vl+1, *vl))) {
-			free(tk);
-			return -1;			/* parse error */
-		}
-		vl++;
-		*cp = '\0';
-	}
-
-	*name = tk;
-	*value = vl;
 	return 0;
 }
 
@@ -90,16 +54,14 @@ fsprobe_get_devname_by_spec(const char *spec)
 		else if (!strcmp(name,"UUID"))
 			nspec = fsprobe_get_devname_by_uuid(value);
 
-		free((void *) name);
+		free(name);
+		free(value);
 		return nspec;
 	}
 
 	return canonicalize_path(spec);
 }
 
-#ifdef HAVE_LIBBLKID
-static blkid_cache blcache;
-
 void
 fsprobe_init(void)
 {
@@ -112,7 +74,7 @@ fsprobe_known_fstype(const char *fstype)
 	return blkid_known_fstype(fstype);
 }
 
-#ifdef HAVE_BLKID_EVALUATE_TAG
+#ifdef HAVE_LIBBLKID_INTERNAL
 /*
  * libblkid from util-linux-ng
  * -- recommended
@@ -132,14 +94,14 @@ fsprobe_exit(void)
  * probing interface
  */
 static char *
-fsprobe_get_value(const char *name, const char *devname)
+fsprobe_get_value(const char *name, const char *devname, int *ambi)
 {
-	int fd;
+	int fd, rc;
 	const char *data = NULL;
 
 	if (!devname || !name)
 		return NULL;
-	fd = open_device(devname);
+	fd = open(devname, O_RDONLY);
 	if (fd < 0)
 		return NULL;
 	if (!blprobe)
@@ -148,13 +110,17 @@ fsprobe_get_value(const char *name, const char *devname)
 		goto done;
 	if (blkid_probe_set_device(blprobe, fd, 0, 0))
 		goto done;
-	if (blkid_probe_set_request(blprobe, BLKID_PROBREQ_LABEL |
-			 BLKID_PROBREQ_UUID | BLKID_PROBREQ_TYPE ))
-		goto done;
-	if (blkid_do_safeprobe(blprobe))
-		goto done;
-	if (blkid_probe_lookup_value(blprobe, name, &data, NULL))
-		goto done;
+
+	blkid_probe_enable_superblocks(blprobe, 1);
+
+	blkid_probe_set_superblocks_flags(blprobe,
+		BLKID_SUBLKS_LABEL | BLKID_SUBLKS_UUID | BLKID_SUBLKS_TYPE);
+
+	rc = blkid_do_safeprobe(blprobe);
+	if (ambi)
+		*ambi = rc == -2 ? 1 : 0;	/* ambivalent probing result */
+	if (!rc)
+		blkid_probe_lookup_value(blprobe, name, &data, NULL);
 done:
 	close(fd);
 	return data ? strdup((char *) data) : NULL;
@@ -163,19 +129,25 @@ done:
 char *
 fsprobe_get_label_by_devname(const char *devname)
 {
-	return fsprobe_get_value("LABEL", devname);
+	return fsprobe_get_value("LABEL", devname, NULL);
 }
 
 char *
 fsprobe_get_uuid_by_devname(const char *devname)
 {
-	return fsprobe_get_value("UUID", devname);
+	return fsprobe_get_value("UUID", devname, NULL);
 }
 
 char *
 fsprobe_get_fstype_by_devname(const char *devname)
 {
-	return fsprobe_get_value("TYPE", devname);
+	return fsprobe_get_value("TYPE", devname, NULL);
+}
+
+char *
+fsprobe_get_fstype_by_devname_ambi(const char *devname, int *ambi)
+{
+	return fsprobe_get_value("TYPE", devname, ambi);
 }
 
 char *
@@ -190,7 +162,7 @@ fsprobe_get_devname_by_label(const char *label)
 	return blkid_evaluate_tag("LABEL", label, &blcache);
 }
 
-#else /* !HAVE_BLKID_EVALUATE_TAG */
+#else /* !HAVE_LIBBLKID_INTERNAL */
 
 /*
  * Classic libblkid (from e2fsprogs) without blkid_evaluate_tag()
@@ -243,6 +215,14 @@ fsprobe_get_fstype_by_devname(const char *devname)
 	return tp;
 }
 
+char *
+fsprobe_get_fstype_by_devname_ambi(const char *devname, int *ambi)
+{
+	if (ambi)
+		*ambi = 0;
+	return fsprobe_get_fstype_by_devname(devname);
+}
+
 char *
 fsprobe_get_label_by_devname(const char *devname)
 {
@@ -261,130 +241,4 @@ fsprobe_get_uuid_by_devname(const char *devname)
 	return blkid_get_tag_value(blcache, "UUID", devname);
 }
 
-#endif /* !HAVE_BLKID_EVALUATE_TAG */
-#else  /* !HAVE_LIBBLKID */
-
-/*
- * libvolume_id from udev
- * -- deprecated
- */
-#include <libvolume_id.h>
-
-enum probe_type {
-	VOLUME_ID_NONE,
-	VOLUME_ID_LABEL,
-	VOLUME_ID_UUID,
-	VOLUME_ID_TYPE,
-};
-
-static char
-*probe(const char *device, enum probe_type type)
-{
-	int fd;
-	uint64_t size;
-	struct volume_id *id;
-	const char *val;
-	char *value = NULL;
-	int retries = 0;
-
-	fd = open_device(devname);
-	if (fd < 0)
-		return NULL;
-	id = volume_id_open_fd(fd);
-	if (!id) {
-		close(fd);
-		return NULL;
-	}
-	if (blkdev_get_size(fd, &size) != 0)
-		size = 0;
-	if (volume_id_probe_all(id, 0, size) == 0) {
-		switch(type) {
-		case VOLUME_ID_LABEL:
-			if (volume_id_get_label(id, &val))
-				value  = strdup(val);
-			break;
-		case VOLUME_ID_UUID:
-			if (volume_id_get_uuid(id, &val))
-				value  = strdup(val);
-			break;
-		case VOLUME_ID_TYPE:
-			if (volume_id_get_type(id, &val))
-				value  = strdup(val);
-			break;
-		default:
-			break;
-		}
-	}
-	volume_id_close(id);
-	close(fd);
-	return value;
-}
-
-void
-fsprobe_init(void)
-{
-}
-
-void
-fsprobe_exit(void)
-{
-}
-
-int
-fsprobe_known_fstype(const char *fstype)
-{
-	if (volume_id_get_prober_by_type(fstype) != NULL)
-		return 1;
-	return 0;
-}
-
-char *
-fsprobe_get_uuid_by_devname(const char *devname)
-{
-	return probe(devname, VOLUME_ID_UUID);
-}
-
-char *
-fsprobe_get_label_by_devname(const char *devname)
-{
-	return probe(devname, VOLUME_ID_LABEL);
-}
-
-char *
-fsprobe_get_fstype_by_devname(const char *devname)
-{
-	return probe(devname, VOLUME_ID_TYPE);
-}
-
-char *
-fsprobe_get_devname_by_uuid(const char *uuid)
-{
-	char dev[PATH_MAX];
-	size_t len;
-
-	if (!uuid)
-		return NULL;
-
-	strcpy(dev, _PATH_DEV_BYUUID "/");
-	len = strlen(_PATH_DEV_BYUUID "/");
-	if (!volume_id_encode_string(uuid, &dev[len], sizeof(dev) - len))
-		return NULL;
-	return canonicalize_path(dev);
-}
-
-char *
-fsprobe_get_devname_by_label(const char *label)
-{
-	char dev[PATH_MAX];
-	size_t len;
-
-	if (!label)
-		return NULL;
-	strcpy(dev, _PATH_DEV_BYLABEL "/");
-	len = strlen(_PATH_DEV_BYLABEL "/");
-	if (!volume_id_encode_string(label, &dev[len], sizeof(dev) - len))
-		return NULL;
-	return canonicalize_path(dev);
-}
-
-#endif /* HAVE_LIBVOLUME_ID  */
+#endif /* !HAVE_LIBBLKID_INTERNAL */
